﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace MVCExample
{
    public class CatController: MvcControlBase<CatModel, ICatView>
    {
        public CatController(CatModel model, ICatView view) 
            : base(model, view) { }

        protected override void WireEvents()
        {
            Model.PropertyChanged += Model_PropertyChanged;
            View.PropertyChanged += View_PropertyChanged;
        }

        protected override void UnwireEvents()
        {
            Model.PropertyChanged -= Model_PropertyChanged;
            View.PropertyChanged -= View_PropertyChanged;
        }

        protected override void SetInitialViewState()
        {
            View.CatAge = Model.Age.ToString();
            View.CatName = Model.Name;
        }

        #region Detect Model Change
        private void Model_OnNameChange()
        {
            View.CatName = Model.Name;
        }

        private void Model_OnAgeChange()
        {
            View.CatAge = Model.Age.ToString();
        }

        private void Model_PropertyChanged(
            object sender,
            PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "Name":
                    Model_OnNameChange();
                    break;
                case "Age":
                    Model_OnAgeChange();
                    break;
                default:
                    throw new ArgumentException("Property not handled: " + e.PropertyName);
            }
        }
        #endregion

        #region Detect View Change
        private int StringToInt(String str, int defaultValue)
        {
            int returnValue = defaultValue;
            int.TryParse(str, out returnValue);
            return returnValue;
        }

        private void View_OnAgeChange()
        {
            Model.Age = StringToInt(View.CatAge, 1) + 1;
        }

        private void View_OnNameChange()
        {
            if (View.CatName == null || View.CatName.Trim().Equals(""))
            {
                View.ErrorName = "Name Cannot Be Null or Empty";
            }
            else
            {
                View.ErrorName = "";
                Model.Name = View.CatName;
            }
        }

        private void View_PropertyChanged(
            object sender,
            PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "ChangeName":
                    View_OnNameChange();
                    break;
                case "IncrementAge":
                    View_OnAgeChange();
                    break;
                default:
                    throw new ArgumentException("Property not handled: " + e.PropertyName);
            }
        }
        #endregion


    }
}
