﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MVCExample
{
    public abstract class MvcControlBase<TModel, TView> : IDisposable
    {
        protected TView m_view;
        protected TModel m_model;

        protected TView View
        {
            get { return m_view; }
            set { m_view = value; }
        }

        protected TModel Model
        {
            get { return m_model; }
            set { m_model = value; }
        }

        protected abstract void WireEvents();
        protected abstract void UnwireEvents();
        protected abstract void SetInitialViewState();

        public MvcControlBase(TModel model, TView view)
        {
            if (model == null || view == null)
            {
                throw new ArgumentException("Neither the model nor the view can be null when constructing the MvcControlBase.");
            }
            m_model = model;
            m_view = view;

            SetInitialViewState();
            WireEvents();
        }

        #region IDisposable Implementation

        private bool isDisposed = false;

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        public void Dispose(bool amDisposing)
        {
            if (amDisposing)
            {
                UnwireEvents();
            }
            isDisposed = true;
        }

        ~MvcControlBase()
        {
            if (!isDisposed) Dispose(false);
        }

        #endregion
    }
}
